@@ -119,10 +119,30 @@ bool ceph_resolve_file_search(const std::string& filename_list,
 md_config_t::md_config_t()
   : cluster("ceph"),
 
-#define OPTION(name, type, def_val) name(def_val),
+#define OPTION_OPT_INT(name, def_val) name(def_val),
+#define OPTION_OPT_LONGLONG(name, def_val) name((1LL) * def_val),
+#define OPTION_OPT_STR(name, def_val) name(def_val),
+#define OPTION_OPT_DOUBLE(name, def_val) name(def_val),
+#define OPTION_OPT_FLOAT(name, def_val) name(def_val),
+#define OPTION_OPT_BOOL(name, def_val) name(def_val),
+#define OPTION_OPT_ADDR(name, def_val) name(def_val),
+#define OPTION_OPT_U32(name, def_val) name(def_val),
+#define OPTION_OPT_U64(name, def_val) name(((uint64_t)1) * def_val),
+#define OPTION_OPT_UUID(name, def_val) name(def_val),
+#define OPTION(name, type, def_val) OPTION_##type(name, def_val)
 #define SUBSYS(name, log, gather)
 #define DEFAULT_SUBSYS(log, gather)
 #include "common/config_opts.h"
+#undef OPTION_OPT_INT
+#undef OPTION_OPT_LONGLONG
+#undef OPTION_OPT_STR
+#undef OPTION_OPT_DOUBLE
+#undef OPTION_OPT_FLOAT
+#undef OPTION_OPT_BOOL
+#undef OPTION_OPT_ADDR
+#undef OPTION_OPT_U32
+#undef OPTION_OPT_U64
+#undef OPTION_OPT_UUID
 #undef OPTION
 #undef SUBSYS
 #undef DEFAULT_SUBSYS
@@ -326,6 +346,7 @@ void md_config_t::_show_config(std::ostream *out, Formatter *f)
       *out << opt->name << " = " << buf << std::endl;
     if (f)
       f->dump_string(opt->name, buf);
+    free(buf);
   }
 }
 
@@ -434,6 +455,8 @@ int md_config_t::parse_option(std::vector<const char*>& args,
 	//	  cout << "subsys " << subsys.get_name(o) << " log " << log << " gather " << gather << std::endl;
 	subsys.set_log_level(o, log);
 	subsys.set_gather_level(o, gather);
+	if (oss)
+	  *oss << "debug_" << subsys.get_name(o) << "=" << log << "/" << gather << " ";
       }
       break;
     }	
@@ -502,7 +525,6 @@ int md_config_t::parse_injectargs(std::vector<const char*>& args,
 				  std::ostream *oss)
 {
   assert(lock.is_locked());
-  std::string val;
   int ret = 0;
   for (std::vector<const char*>::iterator i = args.begin(); i != args.end(); ) {
     int r = parse_option(args, i, oss);
@@ -518,6 +540,13 @@ void md_config_t::apply_changes(std::ostream *oss)
   _apply_changes(oss);
 }
 
+bool md_config_t::_internal_field(const string& s)
+{
+  if (s == "internal_safe_to_start_threads")
+    return true;
+  return false;
+}
+
 void md_config_t::_apply_changes(std::ostream *oss)
 {
   /* Maps observers to the configuration options that they care about which
@@ -535,9 +564,10 @@ void md_config_t::_apply_changes(std::ostream *oss)
   for (changed_set_t::const_iterator c = changed.begin();
        c != changed.end(); ++c) {
     const std::string &key(*c);
-    if ((oss) && (!_get_val(key.c_str(), &bufptr, sizeof(buf)))) {
-      (*oss) << "applying configuration change: " << key << " = '"
-		     << buf << "'\n";
+    if ((oss) &&
+	(!_get_val(key.c_str(), &bufptr, sizeof(buf))) &&
+	!_internal_field(key)) {
+      (*oss) << key << " = '" << buf << "' ";
     }
     pair < obs_map_t::iterator, obs_map_t::iterator >
       range(observers.equal_range(key));
@@ -899,7 +929,7 @@ int md_config_t::set_val_raw(const char *val, const config_option *opt)
 }
 
 static const char *CONF_METAVARIABLES[] =
-  { "cluster", "type", "name", "host", "num", "id" };
+  { "cluster", "type", "name", "host", "num", "id", "pid" };
 static const int NUM_CONF_METAVARIABLES =
       (sizeof(CONF_METAVARIABLES) / sizeof(CONF_METAVARIABLES[0]));
 
@@ -975,6 +1005,8 @@ bool md_config_t::expand_meta(std::string &origval) const
 	  out += name.get_id().c_str();
 	else if (var == "id")
 	  out += name.get_id().c_str();
+	else if (var == "pid")
+	  out += stringify(getpid());
 	else
 	  assert(0); // unreachable
 	found_meta = true;
