@@ -120,6 +120,12 @@ namespace librados
     OP_FAILOK = 2,
   };
 
+  class ObjectOperationCompletion {
+  public:
+    virtual ~ObjectOperationCompletion() {}
+    virtual void handle_completion(int r, bufferlist& outbl) = 0;
+  };
+
   /**
    * These flags apply to the ObjectOperation as a whole.
    *
@@ -154,6 +160,8 @@ namespace librados
     void src_cmpxattr(const std::string& src_oid,
 		      const char *name, int op, uint64_t v);
     void exec(const char *cls, const char *method, bufferlist& inbl);
+    void exec(const char *cls, const char *method, bufferlist& inbl, bufferlist *obl, int *prval);
+    void exec(const char *cls, const char *method, bufferlist& inbl, ObjectOperationCompletion *completion);
     /**
      * Guard operation with a check that object version == ver
      *
@@ -654,6 +662,7 @@ namespace librados
     const std::string& get_pool_name() const;
 
     void locator_set_key(const std::string& key);
+    void set_namespace(const std::string& nspace);
 
     int64_t get_id();
 
