@@ -47,7 +47,8 @@ const string LFNIndex::FILENAME_COOKIE = "long";
 const int LFNIndex::FILENAME_PREFIX_LEN =  FILENAME_SHORT_LEN - FILENAME_HASH_LEN - 
 								FILENAME_COOKIE.size() - 
 								FILENAME_EXTRA;
-void LFNIndex::maybe_inject_failure() {
+void LFNIndex::maybe_inject_failure()
+{
   if (error_injection_enabled) {
     if (current_failure > last_failure &&
 	(((double)(rand() % 10000))/((double)(10000))
@@ -62,15 +63,18 @@ void LFNIndex::maybe_inject_failure() {
 
 /* Public methods */
 
-void LFNIndex::set_ref(std::tr1::shared_ptr<CollectionIndex> ref) {
+void LFNIndex::set_ref(std::tr1::shared_ptr<CollectionIndex> ref)
+{
   self_ref = ref;
 }
 
-int LFNIndex::init() {
+int LFNIndex::init()
+{
   return _init();
 }
 
-int LFNIndex::created(const hobject_t &hoid, const char *path) {
+int LFNIndex::created(const hobject_t &hoid, const char *path)
+{
   WRAP_RETRY(
   vector<string> path_comp;
   string short_name;
@@ -86,7 +90,8 @@ int LFNIndex::created(const hobject_t &hoid, const char *path) {
     );
 }
 
-int LFNIndex::unlink(const hobject_t &hoid) {
+int LFNIndex::unlink(const hobject_t &hoid)
+{
   WRAP_RETRY(
   vector<string> path;
   string short_name;
@@ -130,7 +135,8 @@ int LFNIndex::lookup(const hobject_t &hoid,
   );
 }
 
-int LFNIndex::collection_list(vector<hobject_t> *ls) {
+int LFNIndex::collection_list(vector<hobject_t> *ls)
+{
   return _collection_list(ls);
 }
 
@@ -140,13 +146,15 @@ int LFNIndex::collection_list_partial(const hobject_t &start,
 				      int max_count,
 				      snapid_t seq,
 				      vector<hobject_t> *ls,
-				      hobject_t *next) {
+				      hobject_t *next)
+{
   return _collection_list_partial(start, min_count, max_count, seq, ls, next);
 }
 
 /* Derived class utility methods */
 
-int LFNIndex::fsync_dir(const vector<string> &path) {
+int LFNIndex::fsync_dir(const vector<string> &path)
+{
   maybe_inject_failure();
   int fd = ::open(get_full_path_subdir(path).c_str(), O_RDONLY);
   if (fd < 0)
@@ -451,7 +459,8 @@ int LFNIndex::list_subdirs(const vector<string> &to_list,
   return 0;
 }
 
-int LFNIndex::create_path(const vector<string> &to_create) {
+int LFNIndex::create_path(const vector<string> &to_create)
+{
   maybe_inject_failure();
   int r = ::mkdir(get_full_path_subdir(to_create).c_str(), 0777);
   maybe_inject_failure();
@@ -461,7 +470,8 @@ int LFNIndex::create_path(const vector<string> &to_create) {
     return 0;
 }
 
-int LFNIndex::remove_path(const vector<string> &to_remove) {
+int LFNIndex::remove_path(const vector<string> &to_remove)
+{
   maybe_inject_failure();
   int r = ::rmdir(get_full_path_subdir(to_remove).c_str());
   maybe_inject_failure();
@@ -471,7 +481,8 @@ int LFNIndex::remove_path(const vector<string> &to_remove) {
     return 0;
 }
 
-int LFNIndex::path_exists(const vector<string> &to_check, int *exists) {
+int LFNIndex::path_exists(const vector<string> &to_check, int *exists)
+{
   string full_path = get_full_path_subdir(to_check);
   struct stat buf;
   if (::stat(full_path.c_str(), &buf)) {
@@ -532,7 +543,8 @@ int LFNIndex::remove_attr_path(const vector<string> &path,
   return chain_removexattr(full_path.c_str(), mangled_attr_name.c_str());
 }
   
-string LFNIndex::lfn_generate_object_name_keyless(const hobject_t &hoid) {
+string LFNIndex::lfn_generate_object_name_keyless(const hobject_t &hoid)
+{
   char s[FILENAME_MAX_LEN];
   char *end = s + sizeof(s);
   char *t = s;
@@ -572,7 +584,8 @@ string LFNIndex::lfn_generate_object_name_keyless(const hobject_t &hoid) {
 
 static void append_escaped(string::const_iterator begin,
 			   string::const_iterator end, 
-			   string *out) {
+			   string *out)
+{
   for (string::const_iterator i = begin; i != end; ++i) {
     if (*i == '\\') {
       out->append("\\\\");
@@ -588,7 +601,8 @@ static void append_escaped(string::const_iterator begin,
   }
 }
 
-string LFNIndex::lfn_generate_object_name(const hobject_t &hoid) {
+string LFNIndex::lfn_generate_object_name(const hobject_t &hoid)
+{
   if (index_version == HASH_INDEX_TAG)
     return lfn_generate_object_name_keyless(hoid);
   if (index_version == HASH_INDEX_TAG_2)
@@ -635,7 +649,8 @@ string LFNIndex::lfn_generate_object_name(const hobject_t &hoid) {
   return full_name;
 }
 
-string LFNIndex::lfn_generate_object_name_poolless(const hobject_t &hoid) {
+string LFNIndex::lfn_generate_object_name_poolless(const hobject_t &hoid)
+{
   if (index_version == HASH_INDEX_TAG)
     return lfn_generate_object_name_keyless(hoid);
 
@@ -670,7 +685,8 @@ string LFNIndex::lfn_generate_object_name_poolless(const hobject_t &hoid) {
 int LFNIndex::lfn_get_name(const vector<string> &path, 
 			   const hobject_t &hoid,
 			   string *mangled_name, string *out_path,
-			   int *exists) {
+			   int *exists)
+{
   string subdir_path = get_full_path_subdir(path);
   string full_name = lfn_generate_object_name(hoid);
   int r;
@@ -742,7 +758,8 @@ int LFNIndex::lfn_get_name(const vector<string> &path,
 
 int LFNIndex::lfn_created(const vector<string> &path,
 			  const hobject_t &hoid,
-			  const string &mangled_name) {
+			  const string &mangled_name)
+{
   if (!lfn_is_hashed_filename(mangled_name))
     return 0;
   string full_path = get_full_path(path, mangled_name);
@@ -754,7 +771,8 @@ int LFNIndex::lfn_created(const vector<string> &path,
 
 int LFNIndex::lfn_unlink(const vector<string> &path,
 			 const hobject_t &hoid,
-			 const string &mangled_name) {
+			 const string &mangled_name)
+{
   if (!lfn_is_hashed_filename(mangled_name)) {
     string full_path = get_full_path(path, mangled_name);
     maybe_inject_failure();
@@ -812,7 +830,8 @@ int LFNIndex::lfn_unlink(const vector<string> &path,
 
 int LFNIndex::lfn_translate(const vector<string> &path,
 			    const string &short_name,
-			    hobject_t *out) {
+			    hobject_t *out)
+{
   if (!lfn_is_hashed_filename(short_name)) {
     return lfn_parse_object_name(short_name, out);
   }
@@ -829,11 +848,13 @@ int LFNIndex::lfn_translate(const vector<string> &path,
   return lfn_parse_object_name(long_name, out);
 }
 
-bool LFNIndex::lfn_is_object(const string &short_name) {
+bool LFNIndex::lfn_is_object(const string &short_name)
+{
   return lfn_is_hashed_filename(short_name) || !lfn_is_subdir(short_name, 0);
 }
 
-bool LFNIndex::lfn_is_subdir(const string &name, string *demangled) {
+bool LFNIndex::lfn_is_subdir(const string &name, string *demangled)
+{
   if (name.substr(0, SUBDIR_PREFIX.size()) == SUBDIR_PREFIX) {
     if (demangled)
       *demangled = demangle_path_component(name);
@@ -892,7 +913,8 @@ static int parse_object(const char *s, hobject_t& o)
   return 0;
 }
 
-bool LFNIndex::lfn_parse_object_name_keyless(const string &long_name, hobject_t *out) {
+bool LFNIndex::lfn_parse_object_name_keyless(const string &long_name, hobject_t *out)
+{
   bool r = parse_object(long_name.c_str(), *out);
   int64_t pool = -1;
   pg_t pg;
@@ -989,12 +1011,13 @@ bool LFNIndex::lfn_parse_object_name_poolless(const string &long_name,
   pg_t pg;
   if (coll().is_pg_prefix(pg))
     pool = (int64_t)pg.pool();
-  (*out) = hobject_t(name, key, snap, hash, pool);
+  (*out) = hobject_t(name, key, snap, hash, pool, "");
   return true;
 }
 
 
-bool LFNIndex::lfn_parse_object_name(const string &long_name, hobject_t *out) {
+bool LFNIndex::lfn_parse_object_name(const string &long_name, hobject_t *out)
+{
   string name;
   string key;
   string ns;
@@ -1075,12 +1098,12 @@ bool LFNIndex::lfn_parse_object_name(const string &long_name, hobject_t *out) {
   else
     pool = strtoull(pstring.c_str(), NULL, 16);
 
-  (*out) = hobject_t(name, key, snap, hash, (int64_t)pool);
-  out->nspace = ns;
+  (*out) = hobject_t(name, key, snap, hash, (int64_t)pool, ns);
   return true;
 }
 
-bool LFNIndex::lfn_is_hashed_filename(const string &name) {
+bool LFNIndex::lfn_is_hashed_filename(const string &name)
+{
   if (name.size() < (unsigned)FILENAME_SHORT_LEN) {
     return 0;
   }
@@ -1092,7 +1115,8 @@ bool LFNIndex::lfn_is_hashed_filename(const string &name) {
   }
 }
 
-bool LFNIndex::lfn_must_hash(const string &long_name) {
+bool LFNIndex::lfn_must_hash(const string &long_name)
+{
   return (int)long_name.size() >= FILENAME_SHORT_LEN;
 }
 
@@ -1138,16 +1162,16 @@ void LFNIndex::build_filename(const char *old_filename, int i, char *filename, i
 
   hash_filename(old_filename, hash, sizeof(hash));
   int ofs = FILENAME_PREFIX_LEN;
-  int suffix_len;
   while (1) {
-    suffix_len = sprintf(filename + ofs, "_%s_%d_%s", hash, i, FILENAME_COOKIE.c_str());
+    int suffix_len = sprintf(filename + ofs, "_%s_%d_%s", hash, i, FILENAME_COOKIE.c_str());
     if (ofs + suffix_len <= FILENAME_SHORT_LEN || !ofs)
       break;
     ofs--;
   }
 }
 
-string LFNIndex::lfn_get_short_name(const hobject_t &hoid, int i) {
+string LFNIndex::lfn_get_short_name(const hobject_t &hoid, int i)
+{
   string long_name = lfn_generate_object_name(hoid);
   assert(lfn_must_hash(long_name));
   char buf[FILENAME_SHORT_LEN + 4];
@@ -1155,11 +1179,13 @@ string LFNIndex::lfn_get_short_name(const hobject_t &hoid, int i) {
   return string(buf);
 }
 
-const string &LFNIndex::get_base_path() {
+const string &LFNIndex::get_base_path()
+{
   return base_path;
 }
 
-string LFNIndex::get_full_path_subdir(const vector<string> &rel) {
+string LFNIndex::get_full_path_subdir(const vector<string> &rel)
+{
   string retval = get_base_path();
   for (vector<string>::const_iterator i = rel.begin();
        i != rel.end();
@@ -1170,20 +1196,24 @@ string LFNIndex::get_full_path_subdir(const vector<string> &rel) {
   return retval;
 }
 
-string LFNIndex::get_full_path(const vector<string> &rel, const string &name) {
+string LFNIndex::get_full_path(const vector<string> &rel, const string &name)
+{
   return get_full_path_subdir(rel) + "/" + name;
 }
 
-string LFNIndex::mangle_path_component(const string &component) {
+string LFNIndex::mangle_path_component(const string &component)
+{
   return SUBDIR_PREFIX + component;
 }
 
-string LFNIndex::demangle_path_component(const string &component) {
+string LFNIndex::demangle_path_component(const string &component)
+{
   return component.substr(SUBDIR_PREFIX.size(), component.size() - SUBDIR_PREFIX.size());
 }
 
 int LFNIndex::decompose_full_path(const char *in, vector<string> *out,
-				  hobject_t *hoid, string *shortname) {
+				  hobject_t *hoid, string *shortname)
+{
   const char *beginning = in + get_base_path().size();
   const char *end = beginning;
   while (1) {
@@ -1206,6 +1236,7 @@ int LFNIndex::decompose_full_path(const char *in, vector<string> *out,
   return 0;
 }
 
-string LFNIndex::mangle_attr_name(const string &attr) {
+string LFNIndex::mangle_attr_name(const string &attr)
+{
   return PHASH_ATTR_PREFIX + attr;
 }
